@@ -5,8 +5,6 @@
 #include <pc.h>
 #include <dpmi.h>
 #include <bios.h>
-#include <go32.h>
-#include <libc/dosio.h>
 #include <sys/exceptn.h>
 #include <sys/movedata.h>
 
@@ -16,7 +14,6 @@
 
 #undef read
 #undef write
-#undef pipe
 #undef close
 #undef select
 
@@ -40,18 +37,21 @@ static void sigbreak(int sig)
 	break_pressed = 1;
 }
 
-void get_terminal_size(int *x, int *y)
+static unsigned screen_x = 80;
+static unsigned screen_y = 25;
+
+int get_terminal_size(int fd, int *x, int *y)
 {
-	*x = ScreenCols();
-	*y = ScreenRows();
+	*x = screen_x = ScreenCols();
+	*y = screen_y = ScreenRows();
+	return 0;
 }
 
-void handle_terminal_resize(void (*fn)(int, int), int *x, int *y)
+void handle_terminal_resize(int fd, void (*fn)(void))
 {
-	get_terminal_size(x, y);
 }
 
-void unhandle_terminal_resize(void)
+void unhandle_terminal_resize(int fd)
 {
 }
 
@@ -75,7 +75,7 @@ static unsigned char dos_mouse_buttons;
 static int dos_mouse_last_x;
 static int dos_mouse_last_y;
 static int dos_mouse_last_button;
-static uttime dos_mouse_time;
+static ttime dos_mouse_time;
 
 static struct links_event *dos_mouse_queue = DUMMY;
 static int dos_mouse_queue_n;
@@ -150,7 +150,6 @@ static void dos_mouse_enqueue(int x, int y, int b)
 		dos_mouse_queue_n--;
 		goto set_last;
 	}
-	if ((unsigned)dos_mouse_queue_n > (unsigned)MAXINT / sizeof(struct links_event) - 1) overalloc();
 	dos_mouse_queue = mem_realloc(dos_mouse_queue, (dos_mouse_queue_n + 1) * sizeof(struct links_event));
 set_last:
 	dos_mouse_queue[dos_mouse_queue_n].ev = EV_MOUSE;
@@ -177,7 +176,7 @@ static void dos_mouse_poll(void)
 	__dpmi_regs r;
 	dos_poll_break();
 	if (dos_mouse_initialized) {
-		if (dos_mouse_initialized == 1 && get_time() - dos_mouse_time > DOS_MOUSE_FLUSH_TIME)
+		if (dos_mouse_initialized == 1 && (uttime)get_time() - (uttime)dos_mouse_time > DOS_MOUSE_FLUSH_TIME)
 			dos_mouse_initialized = 2;
 		for (i = 0; i < dos_mouse_buttons; i++) {
 			if (dos_mouse_initialized == 1 && i > 0) {
@@ -238,15 +237,13 @@ x:
 
 void *handle_mouse(int cons, void (*fn)(void *, unsigned char *, int), void *data)
 {
-	int x, y;
-	get_terminal_size(&x, &y);
-	dos_mouse_init(x * 8, y * 8);
+	dos_mouse_init(screen_backbuffer_x * 8, screen_backbuffer_y * 8);
 	if (!dos_mouse_initialized) return NULL;
 	dos_mouse_show();
 	txt_mouse_handler = fn;
 	txt_mouse_data = data;
 
-	return DUMMY;
+	return (void *)1;
 }
 
 void unhandle_mouse(void *data)
@@ -286,10 +283,9 @@ static int dos_mouse_event(void)
 	return 0;
 }
 
-void save_terminal(void)
+void dos_save_screen(void)
 {
 	unsigned char *sc;
-	want_draw();
 	screen_backbuffer_x = ScreenCols();
 	screen_backbuffer_y = ScreenRows();
 	screen_default_attr = ScreenAttrib;
@@ -302,12 +298,10 @@ void save_terminal(void)
 	ScreenRetrieve(sc);
 	ScreenGetCursor(&saved_cursor_y, &saved_cursor_x);
 	screen_backbuffer = sc;
-	done_draw();
 }
 
-void restore_terminal(void)
+void dos_restore_screen(void)
 {
-	want_draw();
 	if (screen_backbuffer) {
 		unsigned char *sc;
 		if (ScreenCols() == screen_backbuffer_x && ScreenRows() == screen_backbuffer_y) {
@@ -318,7 +312,6 @@ void restore_terminal(void)
 		screen_backbuffer = NULL;
 		mem_free(sc);
 	}
-	done_draw();
 }
 
 static void ansi_initialize(void)
@@ -326,6 +319,8 @@ static void ansi_initialize(void)
 	if (screen_initialized)
 		return;
 
+	dos_save_screen();
+
 	ScreenClear();
 	cursor_x = 0;
 	cursor_y = 0;
@@ -339,6 +334,7 @@ static void ansi_terminate(void)
 		return;
 
 	ScreenSetCursor(0, 0);
+	dos_restore_screen();
 	screen_initialized = 0;
 }
 
@@ -369,7 +365,6 @@ process_ansi:
 				continue;
 			case 10:
 				cursor_y++;
-				/*-fallthrough*/
 			case 13:
 				cursor_x = 0;
 				break;
@@ -379,7 +374,6 @@ process_ansi:
 					case ')':
 						if (!size--) goto ret;
 						str++;
-						continue;
 					case '7':
 					default:
 						continue;
@@ -414,8 +408,8 @@ process_ansi:
 					case 'm':
 						while (buffer[0] >= '0' && buffer[0] < '9') {
 							unsigned long a;
-							char *e;
-							a = strtoul(cast_const_char buffer, &e, 10);
+							unsigned char *e;
+							a = strtoul(cast_const_char buffer, (char **)(void *)&e, 10);
 							if (*e == ';') e++;
 							memmove(buffer, e, strlen(cast_const_char e) + 1);
 							switch (a) {
@@ -518,13 +512,19 @@ static inline void pipe_wake(void)
 
 #include "vpipe.inc"
 
-int dos_pipe(int fd[2])
+int c_pipe(int *fd)
 {
 	int r = vpipe_create(fd);
-	/*printf("dos_pipe: (%d) : %d,%d\n", r, fd[0], fd[1]);*/
+	/*printf("c_pipe: (%d) : %d,%d\n", r, fd[0], fd[1]);*/
 	return r;
 }
 
+void set_nonblock(int fd)
+{
+	int rs;
+	EINTRLOOP(rs, fcntl(fd, F_SETFL, O_NONBLOCK));
+}
+
 int dos_read(int fd, void *buf, size_t size)
 {
 	int r;
@@ -754,7 +754,7 @@ void setcooked(int ctl)
 
 #define RANDOM_POOL_SIZE	65536
 
-void os_seed_random(unsigned char **pool, int *pool_size)
+void os_seed_random(unsigned char **pool, unsigned *pool_size)
 {
 	unsigned *random_pool, *tmp_pool;
 	int a, i;
@@ -775,16 +775,6 @@ void init_os(void)
 	int s, rs;
 	struct sigaction sa;
 
-#ifdef _STAT_INODE
-	_djstat_flags |= _STAT_INODE;
-#endif
-#ifdef _STAT_EXEC_EXT
-	_djstat_flags |= _STAT_EXEC_EXT;
-#endif
-#ifdef _STAT_EXEC_MAGIC
-	_djstat_flags |= _STAT_EXEC_MAGIC;
-#endif
-
 	init_seq_len = strlen(cast_const_char init_seq);
 
 	/* preload the packet driver */
@@ -808,43 +798,6 @@ void terminate_osdep(void)
 		mem_free(screen_backbuffer);
 }
 
-int os_default_language(void)
-{
-	__dpmi_regs r;
-	memset(&r, 0, sizeof r);
-	r.x.ax = 0x3800;
-	r.x.dx = __tb_offset;
-	r.x.ds = __tb_segment;
-	__dpmi_int(0x21, &r);
-	if (!(r.x.flags & 1)) {
-		return get_country_language(r.x.bx);
-	}
-	return -1;
-}
-
-int os_default_charset(void)
-{
-	__dpmi_regs r;
-	memset(&r, 0, sizeof r);
-	r.x.ax = 0x6601;
-	__dpmi_int(0x21, &r);
-	if (!(r.x.flags & 1)) {
-		unsigned char a[8];
-		int cp;
-		snprintf(cast_char a, sizeof a, "%d", r.x.bx);
-		if ((cp = get_cp_index(a)) >= 0 && cp != utf8_table)
-			return cp;
-	}
-	return 0;
-}
-
-int dos_is_bw(void)
-{
-	unsigned char cfg;
-	dosmemget(0x410, sizeof cfg, &cfg);
-	return (cfg & 0x30) == 0x30;
-}
-
 #else
 
 typedef int dos_c_no_empty_unit;
